/*  Note: You are free to use whatever license you want.
    Eventually you will be able to edit it within Glade. */

/*  gladeedit
 *  Copyright (C) <YEAR> <AUTHORS>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gladesig.h"
#include "gladesrc.h"

GtkWidget*
get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
void
add_pixmap_directory                   (gchar           *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
gchar*
check_file_exists                      (gchar           *directory,
                                        gchar           *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        gchar           *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = check_file_exists ("pixmaps", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

GtkWidget*
create_main_window ()
{
  GtkWidget *main_window;
  GtkWidget *vbox1;
  GtkWidget *menubar1;
  GtkWidget *File;
  GtkWidget *File_menu;
  GtkWidget *New;
  GtkWidget *Open;
  GtkWidget *Save;
  GtkWidget *Save_As;
  GtkWidget *separator1;
  GtkWidget *Quit;
  GtkWidget *Edit;
  GtkWidget *Edit_menu;
  GtkWidget *Cut;
  GtkWidget *Copy;
  GtkWidget *Paste;
  GtkWidget *Delete;
  GtkWidget *View;
  GtkWidget *View_menu;
  GtkWidget *Font;
  GtkWidget *Help;
  GtkWidget *Help_menu;
  GtkWidget *About;
  GtkWidget *toolbar1;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *scrolledwindow1;
  GtkWidget *text1;
  GtkWidget *statusbar1;
  GtkAccelGroup *accel_group;

  main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (main_window), "main_window", main_window);
  gtk_widget_set_usize (main_window, 600, 450);
  gtk_signal_connect (GTK_OBJECT (main_window), "delete_event",
                      GTK_SIGNAL_FUNC (on_main_window_delete_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (main_window), "The Editor");
  gtk_window_set_policy (GTK_WINDOW (main_window), TRUE, TRUE, FALSE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (main_window), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (main_window), vbox1);

  menubar1 = gtk_menu_bar_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "menubar1", menubar1);
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, TRUE, 0);

  File = gtk_menu_item_new_with_label ("File");
  gtk_object_set_data (GTK_OBJECT (main_window), "File", File);
  gtk_widget_show (File);
  gtk_container_add (GTK_CONTAINER (menubar1), File);

  File_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "File_menu", File_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (File), File_menu);

  New = gtk_menu_item_new_with_label ("New");
  gtk_object_set_data (GTK_OBJECT (main_window), "New", New);
  gtk_widget_show (New);
  gtk_container_add (GTK_CONTAINER (File_menu), New);
  gtk_signal_connect (GTK_OBJECT (New), "activate",
                      GTK_SIGNAL_FUNC (on_New_activate),
                      NULL);
  accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW (main_window), accel_group);
  gtk_widget_add_accelerator (New, "activate", accel_group,
                              GDK_N, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  Open = gtk_menu_item_new_with_label ("Open...");
  gtk_object_set_data (GTK_OBJECT (main_window), "Open", Open);
  gtk_widget_show (Open);
  gtk_container_add (GTK_CONTAINER (File_menu), Open);
  gtk_signal_connect (GTK_OBJECT (Open), "activate",
                      GTK_SIGNAL_FUNC (on_Open_activate),
                      NULL);
  gtk_widget_add_accelerator (Open, "activate", accel_group,
                              GDK_O, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  Save = gtk_menu_item_new_with_label ("Save");
  gtk_object_set_data (GTK_OBJECT (main_window), "Save", Save);
  gtk_widget_show (Save);
  gtk_container_add (GTK_CONTAINER (File_menu), Save);
  gtk_signal_connect (GTK_OBJECT (Save), "activate",
                      GTK_SIGNAL_FUNC (on_Save_activate),
                      NULL);
  gtk_widget_add_accelerator (Save, "activate", accel_group,
                              GDK_S, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  Save_As = gtk_menu_item_new_with_label ("Save As...");
  gtk_object_set_data (GTK_OBJECT (main_window), "Save_As", Save_As);
  gtk_widget_show (Save_As);
  gtk_container_add (GTK_CONTAINER (File_menu), Save_As);
  gtk_signal_connect (GTK_OBJECT (Save_As), "activate",
                      GTK_SIGNAL_FUNC (on_Save_As_activate),
                      NULL);

  separator1 = gtk_menu_item_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "separator1", separator1);
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (File_menu), separator1);

  Quit = gtk_menu_item_new_with_label ("Quit");
  gtk_object_set_data (GTK_OBJECT (main_window), "Quit", Quit);
  gtk_widget_show (Quit);
  gtk_container_add (GTK_CONTAINER (File_menu), Quit);
  gtk_signal_connect (GTK_OBJECT (Quit), "activate",
                      GTK_SIGNAL_FUNC (on_Quit_activate),
                      NULL);
  gtk_widget_add_accelerator (Quit, "activate", accel_group,
                              GDK_Q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  Edit = gtk_menu_item_new_with_label ("Edit");
  gtk_object_set_data (GTK_OBJECT (main_window), "Edit", Edit);
  gtk_widget_show (Edit);
  gtk_container_add (GTK_CONTAINER (menubar1), Edit);

  Edit_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "Edit_menu", Edit_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (Edit), Edit_menu);

  Cut = gtk_menu_item_new_with_label ("Cut");
  gtk_object_set_data (GTK_OBJECT (main_window), "Cut", Cut);
  gtk_widget_show (Cut);
  gtk_container_add (GTK_CONTAINER (Edit_menu), Cut);
  gtk_signal_connect (GTK_OBJECT (Cut), "activate",
                      GTK_SIGNAL_FUNC (on_Cut_activate),
                      NULL);
  gtk_widget_add_accelerator (Cut, "activate", accel_group,
                              GDK_X, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  Copy = gtk_menu_item_new_with_label ("Copy");
  gtk_object_set_data (GTK_OBJECT (main_window), "Copy", Copy);
  gtk_widget_show (Copy);
  gtk_container_add (GTK_CONTAINER (Edit_menu), Copy);
  gtk_signal_connect (GTK_OBJECT (Copy), "activate",
                      GTK_SIGNAL_FUNC (on_Copy_activate),
                      NULL);
  gtk_widget_add_accelerator (Copy, "activate", accel_group,
                              GDK_C, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  Paste = gtk_menu_item_new_with_label ("Paste");
  gtk_object_set_data (GTK_OBJECT (main_window), "Paste", Paste);
  gtk_widget_show (Paste);
  gtk_container_add (GTK_CONTAINER (Edit_menu), Paste);
  gtk_signal_connect (GTK_OBJECT (Paste), "activate",
                      GTK_SIGNAL_FUNC (on_Paste_activate),
                      NULL);
  gtk_widget_add_accelerator (Paste, "activate", accel_group,
                              GDK_V, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  Delete = gtk_menu_item_new_with_label ("Delete");
  gtk_object_set_data (GTK_OBJECT (main_window), "Delete", Delete);
  gtk_widget_show (Delete);
  gtk_container_add (GTK_CONTAINER (Edit_menu), Delete);
  gtk_signal_connect (GTK_OBJECT (Delete), "activate",
                      GTK_SIGNAL_FUNC (on_Delete_activate),
                      NULL);
  gtk_widget_add_accelerator (Delete, "activate", accel_group,
                              GDK_Delete, 0, GTK_ACCEL_VISIBLE);

  View = gtk_menu_item_new_with_label ("View");
  gtk_object_set_data (GTK_OBJECT (main_window), "View", View);
  gtk_widget_show (View);
  gtk_container_add (GTK_CONTAINER (menubar1), View);

  View_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "View_menu", View_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (View), View_menu);

  Font = gtk_menu_item_new_with_label ("Change Font...");
  gtk_object_set_data (GTK_OBJECT (main_window), "Font", Font);
  gtk_widget_show (Font);
  gtk_container_add (GTK_CONTAINER (View_menu), Font);
  gtk_signal_connect (GTK_OBJECT (Font), "activate",
                      GTK_SIGNAL_FUNC (on_Font_activate),
                      NULL);

  Help = gtk_menu_item_new_with_label ("Help");
  gtk_object_set_data (GTK_OBJECT (main_window), "Help", Help);
  gtk_widget_show (Help);
  gtk_container_add (GTK_CONTAINER (menubar1), Help);

  Help_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "Help_menu", Help_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (Help), Help_menu);

  About = gtk_menu_item_new_with_label ("About...");
  gtk_object_set_data (GTK_OBJECT (main_window), "About", About);
  gtk_widget_show (About);
  gtk_container_add (GTK_CONTAINER (Help_menu), About);
  gtk_signal_connect (GTK_OBJECT (About), "activate",
                      GTK_SIGNAL_FUNC (on_About_activate),
                      NULL);

  toolbar1 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_object_set_data (GTK_OBJECT (main_window), "toolbar1", toolbar1);
  gtk_widget_show (toolbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, TRUE, 0);

  tmp_toolbar_icon = create_pixmap (main_window, "new.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  button1 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "New",
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main_window), "button1", button1);
  gtk_widget_show (button1);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (on_new_button_clicked),
                      NULL);

  tmp_toolbar_icon = create_pixmap (main_window, "open.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  button2 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Open",
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main_window), "button2", button2);
  gtk_widget_show (button2);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (on_open_button_clicked),
                      NULL);

  tmp_toolbar_icon = create_pixmap (main_window, "save.xpm");
  if (tmp_toolbar_icon == NULL)
    g_error ("Couldn't create pixmap");
  button3 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                "Save",
                                NULL, NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main_window), "button3", button3);
  gtk_widget_show (button3);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
                      GTK_SIGNAL_FUNC (on_save_button_clicked),
                      NULL);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main_window), "scrolledwindow1", scrolledwindow1);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  text1 = gtk_text_new (NULL, NULL);
  gtk_object_set_data (GTK_OBJECT (main_window), "text1", text1);
  gtk_widget_show (text1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), text1);
  gtk_signal_connect (GTK_OBJECT (text1), "changed",
                      GTK_SIGNAL_FUNC (on_text_changed),
                      NULL);
  gtk_text_set_editable (GTK_TEXT (text1), TRUE);

  statusbar1 = gtk_statusbar_new ();
  gtk_object_set_data (GTK_OBJECT (main_window), "statusbar1", statusbar1);
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, TRUE, 0);

  return main_window;
}

GtkWidget*
create_open_file_selection ()
{
  GtkWidget *open_file_selection;
  GtkWidget *ok_button1;
  GtkWidget *cancel_button1;

  open_file_selection = gtk_file_selection_new ("Open File");
  gtk_object_set_data (GTK_OBJECT (open_file_selection), "open_file_selection", open_file_selection);
  gtk_container_border_width (GTK_CONTAINER (open_file_selection), 10);
  gtk_signal_connect (GTK_OBJECT (open_file_selection), "delete_event",
                      GTK_SIGNAL_FUNC (on_open_filesel_delete_event),
                      NULL);
  gtk_window_position (GTK_WINDOW (open_file_selection), GTK_WIN_POS_MOUSE);

  ok_button1 = GTK_FILE_SELECTION (open_file_selection)->ok_button;
  gtk_object_set_data (GTK_OBJECT (open_file_selection), "ok_button1", ok_button1);
  gtk_widget_show (ok_button1);
  GTK_WIDGET_SET_FLAGS (ok_button1, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (ok_button1), "clicked",
                      GTK_SIGNAL_FUNC (on_open_filesel_ok_button_clicked),
                      NULL);

  cancel_button1 = GTK_FILE_SELECTION (open_file_selection)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (open_file_selection), "cancel_button1", cancel_button1);
  gtk_widget_show (cancel_button1);
  GTK_WIDGET_SET_FLAGS (cancel_button1, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (cancel_button1), "clicked",
                      GTK_SIGNAL_FUNC (on_open_filesel_cancel_button_clicked),
                      NULL);

  return open_file_selection;
}

GtkWidget*
create_font_selection ()
{
  GtkWidget *font_selection;
  GtkWidget *ok_button2;
  GtkWidget *cancel_button2;
  GtkWidget *apply_button1;

  font_selection = gtk_font_selection_dialog_new ("Select Font");
  gtk_object_set_data (GTK_OBJECT (font_selection), "font_selection", font_selection);
  gtk_container_border_width (GTK_CONTAINER (font_selection), 4);
  gtk_signal_connect (GTK_OBJECT (font_selection), "delete_event",
                      GTK_SIGNAL_FUNC (on_fontsel_delete_event),
                      NULL);
  gtk_window_position (GTK_WINDOW (font_selection), GTK_WIN_POS_MOUSE);
  gtk_window_set_policy (GTK_WINDOW (font_selection), FALSE, TRUE, TRUE);

  ok_button2 = GTK_FONT_SELECTION_DIALOG (font_selection)->ok_button;
  gtk_object_set_data (GTK_OBJECT (font_selection), "ok_button2", ok_button2);
  gtk_widget_show (ok_button2);
  GTK_WIDGET_SET_FLAGS (ok_button2, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (ok_button2), "clicked",
                      GTK_SIGNAL_FUNC (on_fontsel_ok_button_clicked),
                      NULL);

  cancel_button2 = GTK_FONT_SELECTION_DIALOG (font_selection)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (font_selection), "cancel_button2", cancel_button2);
  gtk_widget_show (cancel_button2);
  GTK_WIDGET_SET_FLAGS (cancel_button2, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (cancel_button2), "clicked",
                      GTK_SIGNAL_FUNC (on_fontsel_cancel_button_clicked),
                      NULL);

  apply_button1 = GTK_FONT_SELECTION_DIALOG (font_selection)->apply_button;
  gtk_object_set_data (GTK_OBJECT (font_selection), "apply_button1", apply_button1);
  gtk_widget_show (apply_button1);
  GTK_WIDGET_SET_FLAGS (apply_button1, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (apply_button1), "clicked",
                      GTK_SIGNAL_FUNC (on_fontsel_apply_button_clicked),
                      NULL);

  return font_selection;
}

GtkWidget*
create_about_dialog ()
{
  GtkWidget *about_dialog;
  GtkWidget *dialog_vbox2;
  GtkWidget *label2;
  GtkWidget *dialog_action_area2;
  GtkWidget *button7;

  about_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (about_dialog), "about_dialog", about_dialog);
  gtk_window_set_title (GTK_WINDOW (about_dialog), "About The Editor");
  gtk_window_position (GTK_WINDOW (about_dialog), GTK_WIN_POS_MOUSE);
  gtk_window_set_policy (GTK_WINDOW (about_dialog), TRUE, TRUE, FALSE);

  dialog_vbox2 = GTK_DIALOG (about_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (about_dialog), "dialog_vbox2", dialog_vbox2);
  gtk_widget_show (dialog_vbox2);
  gtk_container_border_width (GTK_CONTAINER (dialog_vbox2), 2);

  label2 = gtk_label_new ("The Editor\n\nv 1.0\n\n(Example program for Glade)");
  gtk_object_set_data (GTK_OBJECT (about_dialog), "label2", label2);
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), label2, TRUE, TRUE, 10);
  gtk_misc_set_padding (GTK_MISC (label2), 12, 0);

  dialog_action_area2 = GTK_DIALOG (about_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (about_dialog), "dialog_action_area2", dialog_action_area2);
  gtk_widget_show (dialog_action_area2);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10);

  button7 = gtk_button_new_with_label ("  OK  ");
  gtk_object_set_data (GTK_OBJECT (about_dialog), "button7", button7);
  gtk_widget_show (button7);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), button7, FALSE, FALSE, 0);
  GTK_WIDGET_SET_FLAGS (button7, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button7);
  gtk_signal_connect (GTK_OBJECT (button7), "clicked",
                      GTK_SIGNAL_FUNC (on_about_ok_clicked),
                      NULL);

  return about_dialog;
}

GtkWidget*
create_save_file_selection ()
{
  GtkWidget *save_file_selection;
  GtkWidget *ok_button3;
  GtkWidget *cancel_button3;

  save_file_selection = gtk_file_selection_new ("Save File");
  gtk_object_set_data (GTK_OBJECT (save_file_selection), "save_file_selection", save_file_selection);
  gtk_container_border_width (GTK_CONTAINER (save_file_selection), 10);
  gtk_signal_connect (GTK_OBJECT (save_file_selection), "delete_event",
                      GTK_SIGNAL_FUNC (on_save_filesel_delete_event),
                      NULL);
  gtk_window_position (GTK_WINDOW (save_file_selection), GTK_WIN_POS_MOUSE);

  ok_button3 = GTK_FILE_SELECTION (save_file_selection)->ok_button;
  gtk_object_set_data (GTK_OBJECT (save_file_selection), "ok_button3", ok_button3);
  gtk_widget_show (ok_button3);
  GTK_WIDGET_SET_FLAGS (ok_button3, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (ok_button3), "clicked",
                      GTK_SIGNAL_FUNC (on_save_filesel_ok_button_clicked),
                      NULL);

  cancel_button3 = GTK_FILE_SELECTION (save_file_selection)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (save_file_selection), "cancel_button3", cancel_button3);
  gtk_widget_show (cancel_button3);
  GTK_WIDGET_SET_FLAGS (cancel_button3, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (cancel_button3), "clicked",
                      GTK_SIGNAL_FUNC (on_save_filesel_cancel_button_clicked),
                      NULL);

  return save_file_selection;
}

