
/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998-1999  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>

#include <gtk/gtk.h>

#include "gladeconfig.h"


static GDebugKey glade_debug_keys[] = {
  { "warnings", GLADE_DEBUG_WARNINGS },
  { "messages", GLADE_DEBUG_MESSAGES },
};

static guint glade_ndebug_keys = sizeof (glade_debug_keys) / sizeof (GDebugKey);

guint glade_debug_flags = 0;		   /* Global Glade debug flags. */
gboolean glade_debug_messages = FALSE;	   /* If debugging info is output. */


void
glade_debug_init ()
{
  gchar *env_string = NULL;

  env_string = getenv ("GLADE_DEBUG");
  if (env_string != NULL)
    {
      glade_debug_flags = g_parse_debug_string (env_string,
						glade_debug_keys,
						glade_ndebug_keys);
      env_string = NULL;
      if (glade_debug_flags & GLADE_DEBUG_MESSAGES)
	glade_debug_messages = TRUE;
    }

  /* If the GLADE_DEBUG environment variable contains the 'warnings' option,
     we tell GLib to abort() on warnings from Glade, GTK, GDK and GLib.
     This can be handy for finding out exactly where the WARNING messages
     occur, with the help of a debugger. (This only works with GTK 1.1) */
#ifdef GTK_HAVE_FEATURES_1_1_0
  if (glade_debug_flags & GLADE_DEBUG_WARNINGS)
    {
      g_log_set_fatal_mask ("g_log_domain_glib", G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);
      g_log_set_fatal_mask ("Gdk", G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);
      g_log_set_fatal_mask ("Gtk", G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);
      g_log_set_fatal_mask (NULL, G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);
    }
#endif
}
