/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>
#include <locale.h>

#include <gtk/gtk.h>

#include "gladeconfig.h"
#include "gbwidget.h"
#include "glade_newcomp_dialog.h"

struct _GladeClassItem {
  GladeNewComponentDialog * dlg;
  gchar * class_name;
};

typedef struct _GladeClassItem GladeClassItem;

static void glade_new_component_dialog_init (GladeNewComponentDialog* dlg);
static void glade_new_component_dialog_class_init (GladeNewComponentDialogClass *klass);

static void on_ok (GtkWidget * widget, GladeNewComponentDialog * dlg);
static void on_name_changed (GtkWidget * widget, GladeNewComponentDialog *dlg);
static void add_widget_class (gpointer key, gpointer value, gpointer dlg_ptr);
static void on_class_activate (GtkWidget * widget, GladeClassItem * ci);
static void on_class_destroy (GtkWidget * widget, GladeClassItem * ci);

guint
glade_new_component_dialog_get_type ()
{
  static guint the_type = 0;
  if (!the_type)
    {
      GtkTypeInfo the_type_info = 
      {
	"GladeNewComponentDialog",
	sizeof (GladeNewComponentDialog),
	sizeof (GladeNewComponentDialogClass),
	(GtkClassInitFunc) glade_new_component_dialog_class_init,
	(GtkObjectInitFunc) glade_new_component_dialog_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
#ifdef GTK_HAVE_FEATURES_1_1_0
	(GtkClassInitFunc) NULL,
#endif
      };
      the_type = gtk_type_unique (gtk_dialog_get_type (),
				  &the_type_info);
    }
  return the_type;
}

extern GtkWidget *
glade_new_component_dialog_new ()
{
  GladeNewComponentDialog * dlg;
  dlg = (GladeNewComponentDialog*)gtk_type_new (glade_new_component_dialog_get_type ());
  return GTK_WIDGET (dlg);
}

static void
glade_new_component_dialog_class_init (GladeNewComponentDialogClass * klass)
{
  return; /* Nothing todo at the moment. */
}

static void
glade_new_component_dialog_init (GladeNewComponentDialog * dlg)
{
  GtkWidget * table;
  GtkWidget * label;

  /* Dialog setup */
  gtk_window_set_title (GTK_WINDOW (dlg), _("New Component"));

  /* Load a list of classes we can handle. */
  dlg->class_menu = gtk_menu_new ();
  dlg->current_class = NULL;
  g_hash_table_foreach (gb_widget_table, add_widget_class, (gpointer)dlg);

  /* Entry widgets */
  table = gtk_table_new (2, 2, FALSE);
  gtk_table_set_row_spacings (GTK_TABLE (table), 3);
  gtk_table_set_col_spacings (GTK_TABLE (table), 3);
  gtk_container_border_width (GTK_CONTAINER (table), 3);
  gtk_box_pack_start (GTK_BOX (dlg->dialog.vbox), table, FALSE, TRUE, 0);
  gtk_widget_show (table);
  
  label = gtk_label_new ( _("Parent Class:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 0, 1);
  gtk_widget_show (label);

  dlg->parent_class_option = gtk_option_menu_new ();
  gtk_option_menu_set_menu (GTK_OPTION_MENU (dlg->parent_class_option),
			    dlg->class_menu);
  gtk_table_attach_defaults (GTK_TABLE (table), dlg->parent_class_option,
			     1, 2, 0, 1);
  gtk_widget_show (dlg->parent_class_option);

  label = gtk_label_new ( _("Component Name:"));
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  gtk_table_attach_defaults (GTK_TABLE (table), label,
			     0, 1, 1, 2);
  gtk_widget_show (label);

  dlg->component_name_entry = gtk_entry_new ();
  gtk_table_attach_defaults (GTK_TABLE (table), dlg->component_name_entry,
			     1, 2, 1, 2);
  gtk_signal_connect_object (GTK_OBJECT (dlg->component_name_entry), "changed",
			     GTK_SIGNAL_FUNC (on_name_changed),
			     GTK_OBJECT (dlg));
  gtk_widget_show (dlg->component_name_entry);

  /* Button box for our buttons */
  dlg->button_box = gtk_hbutton_box_new ();
  gtk_container_add (GTK_CONTAINER (dlg->dialog.action_area), dlg->button_box);
  gtk_button_box_set_layout (GTK_BUTTON_BOX(dlg->button_box),
			     GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX(dlg->button_box), 5);
  gtk_widget_show (dlg->button_box);

  /* Ok button */
  dlg->ok_button = gtk_button_new_with_label (_("OK"));
  gtk_widget_set_sensitive (dlg->ok_button, FALSE);
  GTK_WIDGET_SET_FLAGS (dlg->ok_button, GTK_CAN_DEFAULT | GTK_HAS_DEFAULT);
  gtk_box_pack_end (GTK_BOX (dlg->button_box), dlg->ok_button, FALSE, TRUE, 5);
  gtk_signal_connect_object (GTK_OBJECT (dlg->ok_button), "clicked",
			     GTK_SIGNAL_FUNC (on_ok),
			     GTK_OBJECT (dlg));
  gtk_widget_show (dlg->ok_button);

  /* Cancel button */
  dlg->cancel_button = gtk_button_new_with_label (_("Cancel"));
  GTK_WIDGET_SET_FLAGS (dlg->cancel_button, GTK_CAN_DEFAULT);
  gtk_box_pack_end (GTK_BOX (dlg->button_box), dlg->cancel_button, FALSE, TRUE,
		    5);
  gtk_signal_connect_object (GTK_OBJECT (dlg->cancel_button), "clicked",
			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (dlg));
  gtk_widget_show (dlg->cancel_button);
}

static void
add_widget_class (gpointer key, gpointer data, gpointer dlg_ptr)
{
  GtkWidget * menuitem;
  GtkWidget * label;
  GtkWidget * pixmap;
  GtkWidget * hbox;
  GladeClassItem * ci;
  GladeNewComponentDialog * dlg = (GladeNewComponentDialog*)dlg_ptr;

  ci = g_new (GladeClassItem, 1);
  ci->dlg = dlg;
  ci->class_name = (gchar*)key;

  menuitem = gtk_menu_item_new ();
  gtk_menu_append (GTK_MENU (dlg->class_menu), menuitem);
 
  label = gtk_label_new ((gchar*)key);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  pixmap = gb_widget_new_gtk_pixmap (menuitem, (gchar*)key);
  hbox = gtk_hbox_new (FALSE, 3);

  gtk_container_add (GTK_CONTAINER (menuitem), hbox);
  gtk_widget_show (hbox);

  if (pixmap)
    {
      gtk_box_pack_start (GTK_BOX (hbox), pixmap, FALSE, FALSE, 0);
      gtk_widget_show (pixmap);
    }

  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
  gtk_widget_show (label);


  gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
		      GTK_SIGNAL_FUNC (on_class_activate), (gpointer)ci);
  gtk_signal_connect (GTK_OBJECT (menuitem), "destroy",
		      GTK_SIGNAL_FUNC (on_class_destroy), (gpointer)ci);

  gtk_widget_show (menuitem);

  if (!dlg->current_class)
    dlg->current_class = ci->class_name;
}

static void
on_ok (GtkWidget * widget, GladeNewComponentDialog * dlg)
{
  gchar * name;

  /* Create new widget/ add to project. */
  name = gtk_entry_get_text (GTK_ENTRY (dlg->component_name_entry));
  /* FIXME ****
  project_new_component (dlg->current_class, name);
  */

  /* Make dialog go away. */
  gtk_widget_destroy (GTK_WIDGET (dlg));
}

static void
on_name_changed (GtkWidget * widget, GladeNewComponentDialog * dlg)
{
  gchar * text = gtk_entry_get_text (GTK_ENTRY (dlg->component_name_entry));
  if (GTK_WIDGET_IS_SENSITIVE (dlg->ok_button))
    {
      if (!text || !text[0])
	gtk_widget_set_sensitive (dlg->ok_button, FALSE);
    }
  else if (text && text[0])
    gtk_widget_set_sensitive (dlg->ok_button, TRUE);
}

static void
on_class_activate (GtkWidget * widget, GladeClassItem * ci)
{
  ci->dlg->current_class = ci->class_name;
}

static void
on_class_destroy (GtkWidget * widget, GladeClassItem * ci)
{
  g_free (ci);
}
