/*  Gtk+ User Interface Builder
 *  Copyright (C) 1998  Damon Chaplin
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>

#include "gladeconfig.h"

#include "glade.h"
#include "gbwidget.h"
#ifdef GTK_HAVE_FEATURES_1_1_0
#include "glade_gtk11lib.h"
#else
#include "glade_gtk10lib.h"
#endif
#include "palette.h"
#include "glade_palette.h"
#include "glade_project.h"


static void palette_create (void);
static void on_palette_select_item (GladePalette *palette,
				    gchar *item_name);


static void
palette_create ()
{
  int index, j;
  GbWidget *gbwidget;
  GdkColormap *colormap;
  GladeWidgetInitData *gwid;
  GladePaletteSectionData *palsec;
  gboolean show_section;

  glade_palette = glade_palette_new ();
  gtk_widget_set_uposition (glade_palette, 0, 250);

  gtk_signal_connect (GTK_OBJECT (glade_palette), "delete_event",
		      GTK_SIGNAL_FUNC (palette_hide), NULL);

  gtk_window_set_title (GTK_WINDOW (glade_palette), _("Palette"));
  gtk_container_border_width (GTK_CONTAINER (glade_palette), 0);

  index = 0;
  colormap = gtk_widget_get_colormap (glade_palette);

  while (1)
    {
      j = 0;
      palsec = &sections[index];
      index++;

      /* If the section name is NULL, we've reached the end of the widgets. */
      if (!palsec->section)
	break;

      show_section = (!strcmp (palsec->section, "NotShown")) ? FALSE : TRUE;

      while (1)
	{
	  gwid = &palsec->widgets[j];
	  j++;

	  /* If the widget name is NULL, we've reached the end of the section.
	   */
	  if (!gwid->name)
	    break;

	  gbwidget = g_hash_table_lookup (gb_widget_table, gwid->name);

	  /* Create the widget's pixmap, if it has one. */
	  if (gbwidget->pixmap_struct && !gbwidget->gdkpixmap)
	    gbwidget->gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &(gbwidget->mask), NULL, gbwidget->pixmap_struct);

	  if (show_section)
	    glade_palette_add_widget (GLADE_PALETTE (glade_palette),
				      palsec->section, gwid->name,
				      gbwidget->gdkpixmap, gbwidget->mask,
				      gbwidget->tooltip);
	}
    }

  gtk_signal_connect (GTK_OBJECT (glade_palette), "select_item",
		      GTK_SIGNAL_FUNC (on_palette_select_item), NULL);
}


void
palette_show (GtkWidget * widget,
	      gpointer data)
{
  if (!glade_palette)
    palette_create ();
  gtk_widget_show (glade_palette);
  /* This maps the window, which also de-iconifies it according to ICCCM. */
  gdk_window_show (GTK_WIDGET (glade_palette)->window);
  gdk_window_raise (GTK_WIDGET (glade_palette)->window);
}


gint
palette_hide (GtkWidget * widget,
	      gpointer data)
{
  gtk_widget_hide (glade_palette);
  return TRUE;
}


static void
on_palette_select_item (GladePalette *palette,
			gchar *item_name)
{
  GtkType type;

  /* Ignore the selector. */
  if (item_name == NULL)
    return;

  /* See if a toplevel item was selected - a window, dialog or menu.
     If it was, we create a new component and reset the palette. */
  type = gtk_type_from_name (item_name);
  g_return_if_fail (type != 0);

  if (gtk_type_is_a (type, gtk_window_get_type ())
      || gtk_type_is_a (type, gtk_menu_get_type ()))
    {
      GtkWidget *component;

      component = glade_project_new_component (current_project, item_name,
					       NULL);
      glade_palette_reset_selection (GLADE_PALETTE (glade_palette), FALSE);
      if (component)
	{
	  glade_project_show_component (current_project, component);
	  gb_widget_show_properties (component);
	}
    }
}
